
    // LocationBuilder.hpp
    // Copyright (C) 2013  Giménez, Christian N.

    // This program is free software: you can redistribute it and/or modify
    // it under the terms of the GNU General Public License as published by
    // the Free Software Foundation, either version 3 of the License, or
    // (at your option) any later version.

    // This program is distributed in the hope that it will be useful,
    // but WITHOUT ANY WARRANTY; without even the implied warranty of
    // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    // GNU General Public License for more details.

    // You should have received a copy of the GNU General Public License
    // along with this program.  If not, see <http://www.gnu.org/licenses/>.

    // Miércoles 23 De Enero Del 2013    


#ifndef __LOCATIONBUILDER_HPP__
#define __LOCATIONBUILDER_HPP__

#include "Location.hpp"
#include "Terminal.hpp"

using namespace Terminal;

namespace Loaders{

  /**
     I'm a builder and I can create Locations with all the necessary! 
  */
  class LocationBuilder {

  protected:
    /**
       This is the Location instance that I'm building.
    */
    Location *location;


  public:
    /**
       Create a new instance of a LocationBuilder.
    */
    LocationBuilder();
    ~LocationBuilder();

    /**
       Return the recently created Location, if you request one.

       Please, use build_location first.
    */
    Location* get_location();
  
    /**
       I create a new instance of a Location. Later you can retrieve it asking me with get_location() message.

       @param name Is a string, this is the name of the new Location instance.
       @param model Is a string that has the model name of the location.
    */
    void build_location(string name, string model);  
    /**
       Add a texture to the model.

       @param texture is the texture name.
    */
    //void build_texture(string texture);
    
    /**
       Add a light into the location recently created.
       
       @param name An instance of string.
       @param dir A float array with three elements representing a Math vector.
       @param color A float array with three elements (RGB color table).
     */
    void build_light(string name, float dir[3], float color[3]);

    //void build_starting(vector3df position);
  };
}

#endif
