
    // TinyOgre.cpp
    // Copyright (C) 2013  Giménez, Christian N.

    // This program is free software: you can redistribute it and/or modify
    // it under the terms of the GNU General Public License as published by
    // the Free Software Foundation, either version 3 of the License, or
    // (at your option) any later version.

    // This program is distributed in the hope that it will be useful,
    // but WITHOUT ANY WARRANTY; without even the implied warranty of
    // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    // GNU General Public License for more details.

    // You should have received a copy of the GNU General Public License
    // along with this program.  If not, see <http://www.gnu.org/licenses/>.

    // Miércoles 23 De Enero Del 2013    


/*
  Taken from OgreWiki, file: TinyOgre.h

  This is a TinyOgre modified.
*/
#include "TinyOgre.h"

#include <OgreLogManager.h>
#include <OgreViewport.h>
#include <OgreConfigFile.h>
#include <OgreEntity.h>
#include <OgreWindowEventUtilities.h>
#include <OgreString.h>
#include <OgreRoot.h>
#include <OgreConfigFile.h>

#include <iostream>

#include "Game.hpp"

using namespace std;

TinyOgre::TinyOgre(void)
    : mRoot(0),
    mCamera(0),
    mSceneMgr(0),
    mWindow(0),
    mResourcesCfg(Ogre::StringUtil::BLANK),
    mPluginsCfg(Ogre::StringUtil::BLANK)
{
  if (mSceneMgr != NULL){
    game = new Game(mSceneMgr);
  }else{
    game = NULL;
  }
}


TinyOgre::~TinyOgre(void)
{
    delete mRoot;
    delete game;
}


bool TinyOgre::go(void)
{
#ifdef _DEBUG
    mResourcesCfg = "resources_d.cfg";
    mPluginsCfg = "plugins_d.cfg";
#else
    mResourcesCfg = "resources.cfg";
    mPluginsCfg = "plugins.cfg";
#endif

    // construct Ogre::Root
    mRoot = new Ogre::Root(mPluginsCfg);


    // setup resources
    // Load resource paths from config file
    Ogre::ConfigFile cf;
    cf.load(mResourcesCfg);

    // Go through all sections & settings in the file
    Ogre::ConfigFile::SectionIterator seci = cf.getSectionIterator();

    Ogre::String secName, typeName, archName;
    while (seci.hasMoreElements())
    {
        secName = seci.peekNextKey();
        Ogre::ConfigFile::SettingsMultiMap *settings = seci.getNext();
        Ogre::ConfigFile::SettingsMultiMap::iterator i;
        for (i = settings->begin(); i != settings->end(); ++i)
        {
            typeName = i->first;
            archName = i->second;
            Ogre::ResourceGroupManager::getSingleton().addResourceLocation(
                archName, typeName, secName);
        }
    }

    // configure
    // Show the configuration dialog and initialise the system
    // You can skip this and use root.restoreConfig() to load configuration
    // settings if you were sure there are valid ones saved in ogre.cfg
    if(mRoot->restoreConfig() || mRoot->showConfigDialog())
    {
        // If returned true, user clicked OK so initialise
        // Here we choose to let the system create a default rendering window by passing 'true'
        mWindow = mRoot->initialise(true, "TinyOgre Render Window");	
    }
    else
    {
        return false;
    }

    load_event_manager();

    // choose scenemanager
    // Get the SceneManager, in this case a generic one
    mSceneMgr = mRoot->createSceneManager(Ogre::ST_GENERIC);

    load_camera();

    load_viewports();
    
    // Set default mipmap level (NB some APIs ignore this)
    Ogre::TextureManager::getSingleton().setDefaultNumMipmaps(5);

    // Create any resource listeners (for loading screens)
    //createResourceListener();

    // load resources
    Ogre::ResourceGroupManager::getSingleton().initialiseAllResourceGroups();

    load_scene();

    load_lights();

    

    iniciar_bucle();



    // We should never be able to reach this corner
    // but return true to calm down our compiler
    return true;
}


Ogre::Camera* TinyOgre::getCamera(){
  return mCamera;
}

Ogre::Root* TinyOgre::getRoot(){
  return mRoot;
}

Ogre::SceneManager* TinyOgre::getSceneMgr(){
  return mSceneMgr;
}

Ogre::RenderWindow* TinyOgre::getWindow(){
  return mWindow;
}

void TinyOgre::terminar(){
  mRoot->shutdown();
  exit(0);
}

void TinyOgre::load_scene(){
  if (game == NULL){
    game = new Game(mSceneMgr);
  }
  game->load();
}

void TinyOgre::load_camera(){
    // create camera
    // Create the camera
    mCamera = mSceneMgr->createCamera("PlayerCam");

    // Position it at 500 in Z direction
    mCamera->setPosition(Ogre::Vector3(0,0,80));
    // Look back along -Z
    mCamera->lookAt(Ogre::Vector3(0,0,-300));
    mCamera->setNearClipDistance(5);
}

void TinyOgre::load_viewports(){
    // create viewports
    // Create one viewport, entire window
    Ogre::Viewport* vp = mWindow->addViewport(mCamera);
    vp->setBackgroundColour(Ogre::ColourValue(0,0,0));

    /**
       @todo No sé hasta que punto este código debería estar aquí. En load_viewports() hay código que manipula la cámara.
       
     */

    // Alter the camera aspect ratio to match the viewport
    mCamera->setAspectRatio(
			    Ogre::Real(vp->getActualWidth()) / Ogre::Real(vp->getActualHeight()));


}

void TinyOgre::load_event_manager(){
    event_manager = new EventManager(this, mWindow);
    event_manager->registrar();
}

void TinyOgre::load_lights(){
    // Set ambient light
    mSceneMgr->setAmbientLight(Ogre::ColourValue(0.5, 0.5, 0.5));

    // Create a light
    Ogre::Light* l = mSceneMgr->createLight("MainLight");
    l->setPosition(20,80,50);
}

bool TinyOgre::iniciar_bucle(){
  while(true)
    {
      // Pump window messages for nice behaviour
      Ogre::WindowEventUtilities::messagePump();
      
      if(mWindow->isClosed()){
	return false;
      }
      
      // Render a frame
      if(!mRoot->renderOneFrame()) return false;
    }

  return true;
}

void TinyOgre::pc_strafe_left(){
  game->pc_strafe_left();
}

void TinyOgre::pc_strafe_right(){
  game->pc_strafe_right();
}

void TinyOgre::pc_forward(){  
  game->pc_forward();
}

void TinyOgre::pc_backwards(){
  game->pc_backwards();
}

void TinyOgre::pc_rot_right(){
  game->pc_rot_right();
}

void TinyOgre::pc_rot_left(){
  game->pc_rot_left();
}

void TinyOgre::cam_toggle_mode(){
  game->camera_toggle_mode();
}
