
    /* TinyOgre.h */
    /* Copyright (C) 2013  Giménez, Christian N. */

    /* This program is free software: you can redistribute it and/or modify */
    /* it under the terms of the GNU General Public License as published by */
    /* the Free Software Foundation, either version 3 of the License, or */
    /* (at your option) any later version. */

    /* This program is distributed in the hope that it will be useful, */
    /* but WITHOUT ANY WARRANTY; without even the implied warranty of */
    /* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
    /* GNU General Public License for more details. */

    /* You should have received a copy of the GNU General Public License */
    /* along with this program.  If not, see <http://www.gnu.org/licenses/>. */

    /* Miércoles 23 De Enero Del 2013     */


/*
-----------------------------------------------------------------------------
Filename:    TinyOgre.h
-----------------------------------------------------------------------------

This source file is part of the
   ___                 __    __ _ _    _ 
  /___\__ _ _ __ ___  / / /\ \ (_) | _(_)
 //  // _` | '__/ _ \ \ \/  \/ / | |/ / |
/ \_// (_| | | |  __/  \  /\  /| |   <| |
\___/ \__, |_|  \___|   \/  \/ |_|_|\_\_|
      |___/                              
      Tutorial Framework
      http://www.ogre3d.org/tikiwiki/
-----------------------------------------------------------------------------
*/
#ifndef __TinyOgre_h_
#define __TinyOgre_h_

#include <OgreRoot.h>
#include <OgreCamera.h>
#include <OgreSceneManager.h>
#include <OgreRenderWindow.h>

#include <OISEvents.h>
#include <OISInputManager.h>
#include <OISKeyboard.h>
#include <OISMouse.h>

#include "EventManager.hpp"
class EventManager;

#include "Game.hpp"

/**
   Mantengo información importante acerca del framework Ogre. Encapsulo esta información y la mantengo disponible para todos.
 */
class TinyOgre
{
protected:
  /**
     @name Atributos para manejar Ogre
  */
  //@{
  Ogre::Root *mRoot;
  Ogre::Camera* mCamera;
  Ogre::SceneManager* mSceneMgr;
  Ogre::RenderWindow* mWindow;
  Ogre::String mResourcesCfg;
  Ogre::String mPluginsCfg;
  //@}
  EventManager *event_manager;

  /**
     @name Game Relative

     Attributes used for the game engine.
   */
  //@{
  Game* game;
  //@}

public:
    TinyOgre(void);
    virtual ~TinyOgre(void);

    /**
       @name Geters y Seters
       Para obtener y cambiar información.
    */
    //@{
    Ogre::Camera *getCamera();
    Ogre::Root *getRoot();
    Ogre::SceneManager *getSceneMgr();
    Ogre::RenderWindow *getWindow();
    //@}

    /**
       @name inicialización y terminación
       
    */
    //@{
    /**
       ¡Inicializo y comienzo!
     */
    bool go(void);
    /**
       Termino y salgo del programa.
     */
    void terminar();
    //@}

    /**
       @name PC movement.
     */
    //@{
    void pc_strafe_left();
    void pc_strafe_right();
    void pc_forward();
    void pc_backwards();
    void pc_rot_right();
    void pc_rot_left();
    //@}
    

    /**
       @name Camera movement.
    */
    //@{
    void cam_toggle_mode();
    //@}


 protected:    
    /**
       @name Carga de elementos
    */
    //@{
    /**
       Carga la escena con todos sus elementos.
       
       Crea las entidades y sus respectivos Ogre::SceneNode's.

       Lo prudente es crear una clase "Mundo" o "Escena" que haga la carga por su parte. Éste método sería implementado por polimorfismo puro, y no haría más que delegar la responsabilidad a esa clase.
     */
    void load_scene();

    /**
       Carga la cámara y la inicializa.

       Es posible que la cámara se implemente en una clase personalizada por el usuario programador. Éste mensaje puede implementarse usando polimorfismo puro y delegar la responsabilidad a una clase "Camara" personalizada.
     */
    void load_camera();

    /**
       Carga los viewports y los inicializa.
       
       Por ahora se usa un sólo viewport.
       
       Es posible que cada viewport se implemente en una clase personalizada por el usuario programador. Éste mensaje puede implementarse usando polimorfismo puro y delegar la responsabilidad a un objeto de clase "Viewport" personalizado.
     */
    void load_viewports();
    
    void load_event_manager();

    void load_lights();
    
    //@}

    /**
       Comienzo a hacer el bucle "pseudoinfinito" que se basa en dibujar/actualizar.
     */
    bool iniciar_bucle();

};

#endif // #ifndef __TinyOgre_h_
